package com.cnbm.dispatch.line1;

import cn.hutool.core.date.DateUtil;
import com.cnbm.dispatch.enums.kuka.step1.Step1Mes2PlcVar;
import com.cnbm.dispatch.enums.kuka.step1.Step1Plc2MesVar;
import com.cnbm.dispatch.enums.kuka.step2.Step2Mes2PlcVar;
import com.cnbm.dispatch.enums.kuka.step2.Step2Plc2MesVar;
import com.cnbm.dispatch.enums.kuka.step3.Step3Mes2PlcVar;
import com.cnbm.dispatch.enums.kuka.step3.Step3Plc2MesVar;
import com.cnbm.packing.service.*;
import com.cnbm.s7.s7connector.enmuc.S7Client;
import com.cnbm.s7.s7connector.type.PlcVar;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import java.text.DecimalFormat;
import java.text.Format;
import java.util.Date;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * @Desc: ""
 * @Author: caixiang
 * @DATE: 2022/12/23 12:33
 */
@Component
@Order(value = 1)
public class KukaEvent3ThreadLine1 implements ApplicationRunner {

    private static final Logger logger = LoggerFactory.getLogger("EVENT3Line1");

    @Autowired
    DynamicDataSourceService service;
    @Autowired
    WoPowerLevelServiceBiz levelServiceBiz;

    @Autowired
    WoCompensationPowerServiceBiz compensationPowerServiceBiz;

    @Autowired
    WoPackagingBoxSubstrateServiceBiz substrateServiceBiz;

    @Autowired
    WoPackagingBoxServiceBiz boxServiceBiz;

    public KukaEvent3ThreadLine1(){
        this.kukaStep3 = Executors.newScheduledThreadPool(1);
    }



    private ScheduledExecutorService kukaStep3;




    /**
     * return
     *          成功： 返回相应的object对象
     *          失败： 返回null
     * */
    private Object read(S7Client s7Client,Step1Plc2MesVar var) {
        try {
            return s7Client.read(var.getArea(), var.getAreaNumber(), var.getByteOffset(), var.getBitOffset(), var.getLength(), var.getStrSize(), var.getType());
        }catch (Throwable e){
            logger.info("host:"+s7Client.getHost()+" ; read 操作出现问题: {}",e.getMessage(),e);
            return null;
        }
    }
    private void write(S7Client s7Client, Step1Mes2PlcVar var, Object newValue) {
        if(var.getType().equals(PlcVar.STRING_Array)){
            String[] s = (String[])newValue;
            String[] ss = (String[])newValue;
            if(s.length > var.getLength() ){
                ss = new String[var.getLength()];
                for(int i=0;i< var.getLength();i++){
                    ss[i] = s[i];
                }
            }
            s7Client.write(var.getArea(), var.getAreaNumber(), var.getByteOffset(), var.getBitOffset(), var.getStrSize(), var.getType(),ss);
        }else {
            s7Client.write(var.getArea(), var.getAreaNumber(), var.getByteOffset(), var.getBitOffset(), var.getStrSize(), var.getType(),newValue);
        }
    }

    private Object read(S7Client s7Client, Step2Plc2MesVar var) {
        try {
            return s7Client.read(var.getArea(), var.getAreaNumber(), var.getByteOffset(), var.getBitOffset(), var.getLength(), var.getStrSize(), var.getType());
        }catch (Throwable e){
            logger.info("host:"+s7Client.getHost()+" ; read 操作出现问题: {}",e.getMessage(),e);
            return null;
        }
    }
    private void write(S7Client s7Client, Step2Mes2PlcVar var, Object newValue) {
        if(var.getType().equals(PlcVar.STRING_Array)){
            String[] s = (String[])newValue;
            String[] ss = (String[])newValue;
            if(s.length > var.getLength() ){
                ss = new String[var.getLength()];
                for(int i=0;i< var.getLength();i++){
                    ss[i] = s[i];
                }
            }
            s7Client.write(var.getArea(), var.getAreaNumber(), var.getByteOffset(), var.getBitOffset(), var.getStrSize(), var.getType(),ss);
        }else {
            s7Client.write(var.getArea(), var.getAreaNumber(), var.getByteOffset(), var.getBitOffset(), var.getStrSize(), var.getType(),newValue);
        }
    }

    private Object read(S7Client s7Client, Step3Plc2MesVar var) {
        try {
            return s7Client.read(var.getArea(), var.getAreaNumber(), var.getByteOffset(), var.getBitOffset(), var.getLength(), var.getStrSize(), var.getType());
        }catch (Throwable e){
            logger.info("host:"+s7Client.getHost()+" ; read 操作出现问题: {}",e.getMessage(),e);
            return null;
        }
    }
    private void write(S7Client s7Client, Step3Mes2PlcVar var, Object newValue) {
        if(var.getType().equals(PlcVar.STRING_Array)){
            String[] s = (String[])newValue;
            String[] ss = (String[])newValue;
            if(s.length > var.getLength() ){
                ss = new String[var.getLength()];
                for(int i=0;i< var.getLength();i++){
                    ss[i] = s[i];
                }
            }
            s7Client.write(var.getArea(), var.getAreaNumber(), var.getByteOffset(), var.getBitOffset(), var.getStrSize(), var.getType(),ss);
        }else {
            s7Client.write(var.getArea(), var.getAreaNumber(), var.getByteOffset(), var.getBitOffset(), var.getStrSize(), var.getType(),newValue);
        }
    }


    public String getBoxId(String sapMaterial){
        Integer todayBoxNum = service.getTodayBoxNum()+1;
        String nowTime = DateUtil.format(new Date(), "yyMMdd");

        todayBoxNum+=500;
        logger.info("BOXID生成Event-------现在生成boxId ，sapMaterial: "+sapMaterial+"，nowTime:"+nowTime+"，todayBoxNum:"+formateString(todayBoxNum));
        String res = "301"+sapMaterial+nowTime+formateString(todayBoxNum);
        return res;
    }


    public String getErrorBoxId(String sapMaterial){

        String nowTime = DateUtil.format(new Date(), "yyMMdd");
        String mixBoxError = "001";
        String res = "444"+sapMaterial+nowTime+mixBoxError;
        logger.info("BOXID生成Event-------现在生成errBoxId ，sapMaterial: "+sapMaterial+"，nowTime:"+nowTime);

        return res;
    }

    public String removeKG(String str){
        return str.substring(0,17);
    }

    public static void main(String[] args) {
        Integer a = 102;
        Format f1 = new DecimalFormat("000");
        System.out.println(f1.format(a));
    }
    public String formateString(Integer a){
        Format f1 = new DecimalFormat("000");
        return f1.format(a);
    }

    //return true 出现混档了，return false 没有出现混档
    private boolean isMixLevel(String[] subIds,Integer size){
        String correctLevel = substrateServiceBiz.getBySubId(subIds[0]).getPowerLevel();
        String errMsg = "以下基板ID出现混档：";
        boolean res = false;
        for(int i=0;i<size;i++){
            String powerLevel = substrateServiceBiz.getBySubId(subIds[i]).getPowerLevel();

            if(!powerLevel.equals(correctLevel)){
                errMsg += "基板ID:"+subIds[i]+" 档位："+powerLevel+", ";
                res = true;
            }
        }
        if(res){
            logger.error(errMsg);
        }

        return res;
    }




    @Override
    public void run(ApplicationArguments args) throws Exception {

        kukaStep3.scheduleAtFixedRate(new Runnable() {
            @Override
            public void run() {
                try {

                    //调度开始
                    Integer shelfIsFull = waitingForTarget(Step3Plc2MesVar.ShelfIsFullArrived, true);
                    logger.info("");
                    logger.info("==================================  现在开始执行 过程三 任务（ 开始 ）   ===================================");
                    if(shelfIsFull != 1){
                        logger.info("装箱单打印Event------失败"+" --- "+"MES监听 Step3Plc2MesVar.ShelfIsFullArrived 是否 等于 1 失败");
                    }else if(shelfIsFull == 2){
                        throw new Exception("MES 监听值出现异常。");
                    }

                    //一. 从plc 中获取 subIdList 和 lineNum
                    String boxId = (String)read(S7Client.S7_DDJ1,Step3Plc2MesVar.BoxId);
                    Integer lineNum = (Integer) read(S7Client.S7_DDJ1,Step3Plc2MesVar.LineNum);
                    if(boxId == null){
                        logger.info("装箱单打印Event-------失败"+" --- "+"从plc里面获取到的boxId 为null");
                        throw new Exception("装箱单打印Event-------boxId 为null");
                    }
                    logger.info("装箱单打印Event------到达站台的boxId :"+boxId);

                    //二. MES 业务
                    //===============           MES 业务 开始         =================
                    //1. box信息录入到box表
                    boxServiceBiz.updateIsArrivedByBoxNo(1,boxId);
                    logger.info("装箱单打印Event------boxId :"+boxId+"更新数据库成功");
                    //===============           MES 业务 结束         =================


                    //四. 当MES完成任务后，把MesToPlc.ShelfIsFullArrivedFinish变量置为true，告诉plc，我操作完成了
                    write(S7Client.S7_DDJ1,Step3Mes2PlcVar.ShelfIsFullArrivedFinish,true);
                    logger.info("装箱单打印Event------boxId :"+boxId+"， ShelfIsFullArrivedFinish置为true 成功");
                    //四. mes 监控 到PlcToMes.SubArrived==false，就把MesToPlc.SubArrivedFinish置为false
                    Integer subArrived2 = waitingForTarget(Step3Plc2MesVar.ShelfIsFullArrived, false);
                    if(subArrived2 != 1){
                        logger.info("装箱单打印Event------失败"+" --- "+"步骤1.  MES监听 Step3Plc2MesVar.ShelfIsFullArrived 是否 等于 1 失败");
                    }else if(subArrived2 == 2){
                        throw new Exception("MES 监听值出现异常。");
                    }
                    logger.info("装箱单打印Event------boxId :"+boxId+"， 监听到subArrived=false 成功");
                    write(S7Client.S7_DDJ1,Step3Mes2PlcVar.ShelfIsFullArrivedFinish,false);
                    logger.info("装箱单打印Event------boxId :"+boxId+"， Even3 成功");

                    logger.info("==================================  现在开始执行 过程三 任务（ 结束 ）   ===================================");
                    logger.info("");
                }catch (Throwable e){
//                    e.printStackTrace();
//                    logger.info("装箱单打印Event 出现异常------"+e.toString());
                    logger.info("装箱单打印Event--- 出现异常 : {}",e.getMessage(),e);
                }

            }
        },1,1, TimeUnit.SECONDS);

    }

    /**
     *
     * arg:
     *      Step1Plc2MesVar 参数类型
     *      targetValue        目的值
     * return:
     *      2 s7 访问异常;
     *      1 成功;
     * */
    private Integer waitingForTarget(Step1Plc2MesVar var, Boolean targetValue) {
        Integer res = 0;
        while (true){
            try {

                Boolean now = (Boolean) read(S7Client.S7_DDJ1,var);
                //注意：即使now为null ，这里也不会抛出异常，这里很容易产生 空轮询。
                if(now == null){
                    throw new Exception("");
                }
                if(targetValue.equals(now)){
                    res = 1;
                    break;
                }
                Thread.sleep(200);
            }catch (Throwable e){
                logger.info("S7 数据采集 异常. 监听 变量 ："+var.getName()+" 为目标值："+targetValue+"失败");
                res = 2;
                break;
            }
        }
        return res;
    }
    /**
     *
     * arg:
     *      Step2Plc2MesVar 参数类型
     *      targetValue        目的值
     * return:
     *      2 s7 访问异常;
     *      1 成功;
     * */
    private Integer waitingForTarget(Step2Plc2MesVar var, Boolean targetValue) {
        Integer res = 0;
        while (true){
            try {
                Boolean now = (Boolean) read(S7Client.S7_DDJ1,var);
                if(now == null){
                    throw new Exception("");
                }
                if(targetValue.equals(now)){
                    res = 1;
                    break;
                }
                Thread.sleep(200);
            }catch (Throwable e){
                logger.info("S7 数据采集 异常. 监听 变量 ："+var.getName()+" 为目标值："+targetValue+"失败");
                res = 2;
                break;
            }
        }
        return res;
    }
    /**
     *
     * arg:
     *      Step3Plc2MesVar 参数类型
     *      targetValue        目的值
     * return:
     *      2 s7 访问异常;
     *      1 成功;
     * */
    private Integer waitingForTarget(Step3Plc2MesVar var, Boolean targetValue) {
        Integer res = 0;
        while (true){
            try {
                Boolean now = (Boolean) read(S7Client.S7_DDJ1,var);
                if(now == null){
                    throw new Exception("");
                }
                if(targetValue.equals(now)){
                    res = 1;
                    break;
                }
                Thread.sleep(200);
            }catch (Throwable e){
                logger.info("S7 数据采集 异常. 监听 变量 ："+var.getName()+" 为目标值："+targetValue+"失败");
                res = 2;
                break;
            }
        }
        return res;
    }
}