package com.cnbm.packing.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.cnbm.admin.utils.BaseSupportUtils;
import com.cnbm.admin.utils.CodeGeneratorHelper;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.common.utils.ConvertUtils;
import com.cnbm.packing.dto.PrintModelDTO;
import com.cnbm.packing.entity.PrintModel;
import com.cnbm.packing.mapper.PrintModelMapper;
import com.cnbm.packing.service.PrintModelServiceBiz;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 打印标签模板表
 *
 * @author codeGenerator
 * @since 2023-02-15
 */
@Service
public class PrintModelServiceBizImpl extends CrudServiceImpl<PrintModelMapper, PrintModel, PrintModelDTO> implements PrintModelServiceBiz {

    @Autowired
    private PrintModelMapper mapper;
    
    @Override
    public QueryWrapper<PrintModel> getWrapper(Map<String, Object> params){
        String name = (String)params.get("name");

        QueryWrapper<PrintModel> wrapper = new QueryWrapper<>();
        wrapper.like(StringUtils.isNotBlank(name), PrintModel.NAME, name);
        if(params.get("type")!=null) {
            Integer type = Integer.parseInt(params.get("type").toString());
            wrapper.eq(ObjectUtils.isNotNull(type), PrintModel.TYPE, type);
        }
        if(params.get("lineBody")!=null) {
            Integer lineBody = Integer.parseInt(params.get("lineBody").toString());
            wrapper.eq(ObjectUtils.isNotNull(lineBody), PrintModel.LINE_BODY, lineBody);
        }
        if(params.get("isEnable")!=null) {
            Integer isEnable = Integer.parseInt(params.get("isEnable").toString());
            wrapper.eq(ObjectUtils.isNotNull(isEnable), PrintModel.IS_ENABLE, isEnable);
        }

        return wrapper;
    }

    @Override
    public PageData<PrintModelDTO> page (Map<String, Object> params){
        IPage<PrintModel> page = baseDao.selectPage(
                getPage(params, PrintModel.CREATE_TIME, false),
                getWrapper(params)
        );
        return getPageData(page, PrintModelDTO.class);
    }

    @Override
    public PrintModelDTO get(Long id) {
        PrintModel entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, PrintModelDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(PrintModelDTO dto) {
        PrintModel entity = ConvertUtils.sourceToTarget(dto, PrintModel.class);
        BaseSupportUtils.setCommonField(entity);
        insert(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(PrintModelDTO dto) {
        PrintModel entity = ConvertUtils.sourceToTarget(dto, PrintModel.class);
        BaseSupportUtils.setUpdateCommonField(entity);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<PrintModelDTO> list() {
        List<PrintModelDTO> list = mapper.list();
        return list;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String getCode() {
        return CodeGeneratorHelper.getPrintModelCode();
    }

    @Override
    public PrintModelDTO getByLineBody(Integer lineBody){
        QueryWrapper<PrintModel> wrapper = new QueryWrapper<>();
        wrapper.eq(ObjectUtils.isNotNull(lineBody), PrintModel.LINE_BODY, lineBody);
        wrapper.eq(PrintModel.IS_ENABLE,1);
        wrapper.eq(PrintModel.TYPE,2);
        List<PrintModel> list = mapper.selectList(wrapper);
        PrintModel entity = new PrintModel();
        if(list.size()>0){
            entity = list.get(0);
        }
        return ConvertUtils.sourceToTarget(entity, PrintModelDTO.class);
    }

}