package com.cnbm.packing.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cnbm.admin.utils.BaseSupportUtils;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.common.utils.ConvertUtils;
import com.cnbm.packing.dto.IdVo;
import com.cnbm.packing.dto.PowerReportDTO;
import com.cnbm.packing.dto.WoPackagingBoxDTO;
import com.cnbm.packing.entity.WoPackagingBox;
import com.cnbm.packing.entity.WoPackagingBoxSubstrate;
import com.cnbm.packing.mapper.WoPackagingBoxMapper;
import com.cnbm.packing.mapper.WoPackagingBoxSubstrateMapper;
import com.cnbm.packing.param.PowerReportQueryParam;
import com.cnbm.packing.param.SubIdQueryParam;
import com.cnbm.packing.service.WoPackagingBoxServiceBiz;
import com.cnbm.packing.service.WoPackagingBoxSubstrateServiceBiz;
import com.cnbm.packing.vo.R;
import com.cnbm.packing.vo.SubIdPageVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 包装箱表
 *
 * @author codeGenerator
 * @since 2023-02-15
 */
@Service
public class WoPackagingBoxServiceBizImpl extends CrudServiceImpl<WoPackagingBoxMapper, WoPackagingBox, WoPackagingBoxDTO> implements WoPackagingBoxServiceBiz {

    @Autowired
    private WoPackagingBoxMapper mapper;
    @Autowired
    private WoPackagingBoxSubstrateMapper substrateMapper;
    @Autowired
    private WoPackagingBoxSubstrateServiceBiz woPackagingBoxSubstrateServiceBiz;


    public static void main(String[] args) {
        LocalDateTime localDate=LocalDateTime.parse("2023-03-23 09:00:00", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        System.out.println(localDate);
    }

    @Override
    public QueryWrapper<WoPackagingBox> getWrapper(Map<String, Object> params){

        QueryWrapper<WoPackagingBox> wrapper = new QueryWrapper<>();

        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        if(params.get("startTime")!=null && params.get("endTime")!=null){
            LocalDateTime startTime = LocalDateTime.parse(params.get("startTime").toString(),df);
            LocalDateTime endTime = LocalDateTime.parse(params.get("endTime").toString(),df);

            wrapper.between(startTime!=null && endTime!=null, WoPackagingBox.CREATE_TIME, startTime, endTime);
        }
        String boxNo = (String) params.get("boxNo");
        wrapper.like(StringUtils.isNotBlank(boxNo), WoPackagingBox.BOX_NO, boxNo);
//        if(params.get("lineBody")!=null) {
//            Integer lineBody = Integer.parseInt(params.get("lineBody").toString());
//            wrapper.eq(ObjectUtils.isNotNull(lineBody), WoPackagingBox.LINE_BODY, lineBody);
//        }
//        if(params.get("printStatus")!=null) {
//            Integer printStatus = Integer.parseInt(params.get("printStatus").toString());
//            wrapper.eq(ObjectUtils.isNotNull(printStatus), WoPackagingBox.PRINT_STATUS, printStatus);
//        }
        if(params.get("model")!=null) {
            Integer model = Integer.parseInt(params.get("model").toString());
            wrapper.eq(ObjectUtils.isNotNull(model), WoPackagingBox.MODEL, model);
        }
        if(params.get("startTime")==null && params.get("endTime")==null && params.get("boxNo")==null){
            wrapper.apply(true, "TO_DAYS(NOW())-TO_DAYS(create_time) = 0");
        }

        return wrapper;
    }

    @Override
    public PageData<WoPackagingBoxDTO> page (Map<String, Object> params){

        IPage<WoPackagingBox> page = baseDao.selectPage(
                getPage(params, WoPackagingBox.CREATE_TIME, false),
                getWrapper(params)
        );
        return getPageData(page, WoPackagingBoxDTO.class);
    }

    @Override
    public WoPackagingBoxDTO get(Long id) {
        WoPackagingBox entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, WoPackagingBoxDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public IdVo add(WoPackagingBoxDTO dto) {
        WoPackagingBox entity = ConvertUtils.sourceToTarget(dto, WoPackagingBox.class);
        BaseSupportUtils.setCommonField(entity);
        insert(entity);
        return IdVo.builder().id(entity.getId()).build();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(WoPackagingBoxDTO dto) {
        WoPackagingBox entity = ConvertUtils.sourceToTarget(dto, WoPackagingBox.class);
        BaseSupportUtils.setUpdateCommonField(entity);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<WoPackagingBoxDTO> list() {
        List<WoPackagingBoxDTO> list = mapper.list();
        return list;
    }

    @Override
    public WoPackagingBoxDTO printList(Long id) {
        WoPackagingBox entity = baseDao.selectById(id);
        WoPackagingBoxDTO dto = ConvertUtils.sourceToTarget(entity, WoPackagingBoxDTO.class);
        matchColor(dto);
        QueryWrapper<WoPackagingBoxSubstrate> wrapper = new QueryWrapper<>();
        wrapper.eq(StringUtils.isNotBlank(entity.getBoxNo()), WoPackagingBoxSubstrate.PACKAGING_BOX_ID, entity.getBoxNo());
        wrapper.orderByAsc(WoPackagingBoxSubstrate.SLOT);
        List<WoPackagingBoxSubstrate> woPackagingBoxSubstrateList = substrateMapper.selectList(wrapper);
        dto.setSubstrateList(woPackagingBoxSubstrateList);
        return dto;
    }

    @Override
    public List<WoPackagingBoxSubstrate> substrateList(Map<String, Object> params) {
        QueryWrapper<WoPackagingBox> boxQueryWrapper = getWrapper(params);
        List<WoPackagingBox> boxList = mapper.selectList(boxQueryWrapper);
        List<WoPackagingBoxSubstrate> substrateList = new ArrayList<>();
        for(WoPackagingBox box: boxList){
            QueryWrapper<WoPackagingBoxSubstrate> substrateQueryWrapper = new QueryWrapper<>();
            substrateQueryWrapper.eq(StringUtils.isNotBlank(box.getBoxNo()), WoPackagingBoxSubstrate.PACKAGING_BOX_ID, box.getBoxNo());
            List<WoPackagingBoxSubstrate> woPackagingBoxSubstrateList = substrateMapper.selectList(substrateQueryWrapper);
            substrateList.addAll(woPackagingBoxSubstrateList);
        }
        return substrateList;
    }

    @Override
    public List<WoPackagingBox> boxList(String woSubstrateId) {
        QueryWrapper<WoPackagingBoxSubstrate> substrateQueryWrapper = new QueryWrapper<>();
        substrateQueryWrapper.eq(StringUtils.isNotBlank(woSubstrateId), WoPackagingBoxSubstrate.WO_SUBSTRATE_ID, woSubstrateId);
        List<WoPackagingBoxSubstrate> substrateList = substrateMapper.selectList(substrateQueryWrapper);
        List<WoPackagingBox> boxList = new ArrayList<>();
        for(WoPackagingBoxSubstrate substrate: substrateList){
            QueryWrapper<WoPackagingBox> boxQueryWrapper = new QueryWrapper<>();
            boxQueryWrapper.like(StringUtils.isNotBlank(substrate.getPackagingBoxId()), WoPackagingBox.BOX_NO, substrate.getPackagingBoxId());
            List<WoPackagingBox> woPackagingBoxList = mapper.selectList(boxQueryWrapper);
            boxList.addAll(woPackagingBoxList);
        }
        return boxList;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public IdVo insertSubstrateManual(WoPackagingBoxDTO dto) {
        //新增包装箱
        WoPackagingBox box = ConvertUtils.sourceToTarget(dto, WoPackagingBox.class);
        BaseSupportUtils.setCommonField(box);
        insert(box);
        //新增模组
        List<WoPackagingBoxSubstrate> substrateList = dto.getSubstrateList();
        for(WoPackagingBoxSubstrate substrate : substrateList){
            //判断模组id是否非空
            if(!StringUtils.isBlank(substrate.getWoSubstrateId())) {
                substrate.setPackagingBoxId(box.getBoxNo());
                substrate.setPowerLevel(box.getPowerLevel());
                substrate.setSapMaterial(box.getSapMaterial());
                substrate.setLineBody(box.getLineBody());
                BaseSupportUtils.setCommonField(substrate);
                woPackagingBoxSubstrateServiceBiz.insert(substrate);
            }
        }
        return IdVo.builder().id(box.getId()).build();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void editSubstrateManual(WoPackagingBoxDTO dto) {
        //获取模组
        List<WoPackagingBoxSubstrate> substrateList = dto.getSubstrateList();
        for(WoPackagingBoxSubstrate substrate : substrateList){
            //新增：id为空 模组非空
            if(ObjectUtils.isEmpty(substrate.getId()) && StringUtils.isNotBlank(substrate.getWoSubstrateId())) {
                substrate.setPackagingBoxId(dto.getBoxNo());
                substrate.setPowerLevel(dto.getPowerLevel());
                substrate.setSapMaterial(dto.getSapMaterial());
                substrate.setLineBody(dto.getLineBody());
                BaseSupportUtils.setCommonField(substrate);
                woPackagingBoxSubstrateServiceBiz.insert(substrate);
            }
            //编辑：id非空 模组非空
            if(ObjectUtils.isNotEmpty(substrate.getId()) && StringUtils.isNotBlank(substrate.getWoSubstrateId())) {
                BaseSupportUtils.setUpdateCommonField(substrate);
                woPackagingBoxSubstrateServiceBiz.updateById(substrate);
            }
            //删除：id非空 模组为空
            if(ObjectUtils.isNotEmpty(substrate.getId()) && StringUtils.isBlank(substrate.getWoSubstrateId())) {
                woPackagingBoxSubstrateServiceBiz.deleteById(substrate.getId());
            }
        }
    }

    @Override
    public int updateIsArrivedByBoxNo(Integer isArrived, String boxNo) {
        return mapper.updateIsArrivedByBoxNo(isArrived,boxNo);
    }

    @Override
    public List<PowerReportDTO> powerReport(PowerReportQueryParam queryParam) {
        return substrateMapper.powerReport(queryParam);
    }

    @Override
    public List<String> orderNameList() {
        return substrateMapper.orderNameList();
    }

    @Override
    public R<IPage<SubIdPageVo>> subIdPage(SubIdQueryParam param) {
        IPage<SubIdPageVo> iPage = substrateMapper.subIdPage(
                new Page<>(param.getCurrent(), param.getSize()),
                param);
        return R.ok(iPage);
    }

    @Override
    public void setColor(List<WoPackagingBoxDTO> list) {
        list.forEach(e -> {
            matchColor(e);
        });
    }
    
    public void matchColor(WoPackagingBoxDTO dto) {
        if (dto.getSapMaterial() != null) {
            String[] sublist = dto.getSapMaterial().split("\\.");
            if (sublist.length > 2) {
                switch (sublist[2]) {
                    case "C0001":
                        dto.setColor("暗灰");
                        break;
                    case "C0002":
                        dto.setColor("暗蓝");
                        break;
                    case "C0003":
                        dto.setColor("暗绿");
                        break;
                    case "C0004":
                        dto.setColor("暗黄");
                        break;
                    case "C0005":
                        dto.setColor("暗橘黄");
                        break;
                    case "C0006":
                        dto.setColor("暗红");
                        break;
                    case "C0007":
                        dto.setColor("亮灰");
                        break;
                    case "C0008":
                        dto.setColor("亮蓝");
                        break;
                    case "C0009":
                        dto.setColor("亮绿");
                        break;
                    case "C0010":
                        dto.setColor("亮黄");
                        break;
                    case "C0011":
                        dto.setColor("亮橘黄");
                        break;
                    case "C0012":
                        dto.setColor("亮红");
                        break;
                    case "C0013":
                        dto.setColor("大红");
                        break;
                    case "C0014":
                        dto.setColor("中国红1185");
                        break;
                    case "C0015":
                        dto.setColor("新蓝825");
                        break;
                    case "C0016":
                        dto.setColor("橄榄绿1186");
                        break;
                    case "C0017":
                        dto.setColor("落日黄3726");
                        break;
                    case "C0018":
                        dto.setColor("玛瑙灰4023");
                        break;
                    case "C0019":
                        dto.setColor("中国红PV-748-R115");
                        break;
                    case "C0020":
                        dto.setColor("阳光橙PV-748-0M2");
                        break;
                    case "C0021":
                        dto.setColor("雪花银PV-748-W815");
                        break;
                    case "C0022":
                        dto.setColor("苍穹灰PV-748-GR960");
                        break;
                    case "C0023":
                        dto.setColor("阳光金PV-748-GL6020");
                        break;
                    case "C0024":
                        dto.setColor("锦缎金PV-748-GL920");
                        break;
                    case "C0025":
                        dto.setColor("森林绿PV-748-G830");
                        break;
                    case "C0026":
                        dto.setColor("恒星绿PV-748-G602");
                        break;
                    case "C0027":
                        dto.setColor("天空蓝PV-748-BL60");
                        break;
                    case "C0028":
                        dto.setColor("香芋紫PV-748-BL40");
                        break;
                    case "C0029":
                        dto.setColor("木槿紫PV-748-RZ92");
                        break;
                    case "C0030":
                        dto.setColor("日光红PV-748-R602");
                        break;
                    case "C0031":
                        dto.setColor("天空蓝BM0020");
                        break;
                    case "C0032":
                        dto.setColor("天蓝色BM0101");
                        break;
                    case "C0033":
                        dto.setColor("亮紫");
                        break;
                    case "C0034":
                        dto.setColor("荒漠迷彩");
                        break;
                    case "C0035":
                        dto.setColor("阳光金");
                        break;
                    case "C0036":
                        dto.setColor("天空蓝");
                        break;
                    case "C0037":
                        dto.setColor("翠绿色");
                        break;
                    case "C0038":
                        dto.setColor("苍穹灰");
                        break;
                    case "C0039":
                        dto.setColor("中国红-3");
                        break;
                    case "C0040":
                        dto.setColor("301中国红亮彩");
                        break;
                    case "C0041":
                        dto.setColor("淡金色");
                        break;
                    case "C0042":
                        dto.setColor("土耳其条纹黄玉大理石");
                        break;
                    case "C0043":
                        dto.setColor("意大利伯林顿灰大理石");
                        break;
                    case "C0044":
                        dto.setColor("黑色");
                        break;
                    case "C0045":
                        dto.setColor("50%透光黑色");
                        break;
                    default:
                        break;
                }
            }
        }
    }
    
}