package com.cnbm.packing.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cnbm.admin.utils.BaseSupportUtils;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.common.utils.ConvertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.AddGroup;
import com.cnbm.common.validator.group.DefaultGroup;
import com.cnbm.packing.dto.ChangePackingBoxDTO;
import com.cnbm.packing.dto.WoPackagingBoxSubstrateDTO;
import com.cnbm.packing.entity.ChangePackagingBoxHistory;
import com.cnbm.packing.entity.WoPackagingBoxSubstrate;
import com.cnbm.packing.mapper.WoPackagingBoxSubstrateMapper;
import com.cnbm.packing.param.PackingInfoQueryParam;
import com.cnbm.packing.service.ChangePackagingBoxHistoryServiceBiz;
import com.cnbm.packing.service.WoPackagingBoxSubstrateServiceBiz;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 包装箱基板关联表
 *
 * @author codeGenerator
 * @since 2023-02-16
 */
@Service
public class WoPackagingBoxSubstrateServiceBizImpl extends CrudServiceImpl<WoPackagingBoxSubstrateMapper, WoPackagingBoxSubstrate, WoPackagingBoxSubstrateDTO> implements WoPackagingBoxSubstrateServiceBiz {

    @Autowired
    private ChangePackagingBoxHistoryServiceBiz changePackagingBoxHistoryService;
    @Autowired
    private WoPackagingBoxSubstrateMapper mapper;

    @Override
    public QueryWrapper<WoPackagingBoxSubstrate> getWrapper(Map<String, Object> params){
        String packagingBoxId = (String) params.get("packagingBoxId");
        String woSubstrateId = (String)params.get("woSubstrateId");

        QueryWrapper<WoPackagingBoxSubstrate> wrapper = new QueryWrapper<>();
        wrapper.like(StringUtils.isNotBlank(packagingBoxId), WoPackagingBoxSubstrate.PACKAGING_BOX_ID, packagingBoxId);
        wrapper.like(StringUtils.isNotBlank(woSubstrateId), WoPackagingBoxSubstrate.WO_SUBSTRATE_ID, woSubstrateId);

        return wrapper;
    }



    @Override
    public PageData<WoPackagingBoxSubstrateDTO> page (Map<String, Object> params){
        IPage<WoPackagingBoxSubstrate> page = baseDao.selectPage(
                getPage(params, WoPackagingBoxSubstrate.CREATE_TIME, false),
                getWrapper(params)
        );
        return getPageData(page, WoPackagingBoxSubstrateDTO.class);
    }

    @Override
    public WoPackagingBoxSubstrateDTO get(Long id) {
        WoPackagingBoxSubstrate entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, WoPackagingBoxSubstrateDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(WoPackagingBoxSubstrateDTO dto) {
        WoPackagingBoxSubstrate entity = ConvertUtils.sourceToTarget(dto, WoPackagingBoxSubstrate.class);
        BaseSupportUtils.setCommonField(entity);
        insert(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(WoPackagingBoxSubstrateDTO dto) {
        WoPackagingBoxSubstrate entity = ConvertUtils.sourceToTarget(dto, WoPackagingBoxSubstrate.class);
        BaseSupportUtils.setUpdateCommonField(entity);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void removeSubstrate(Long id) {
        WoPackagingBoxSubstrate entity = baseDao.selectById(id);
        //移箱写入包装箱拆合箱历史表
        ChangePackagingBoxHistory changePackagingBoxHistory = new ChangePackagingBoxHistory();
        changePackagingBoxHistory.setSubstrateId(entity.getWoSubstrateId());
        changePackagingBoxHistory.setSourceBoxNo(entity.getPackagingBoxId());
        changePackagingBoxHistory.setSourceSlot(entity.getSlot());
        changePackagingBoxHistory.setLeaveTime(LocalDateTime.now());
        changePackagingBoxHistory.setType(2);
        BaseSupportUtils.setCommonField(changePackagingBoxHistory);
        changePackagingBoxHistoryService.insert(changePackagingBoxHistory);
        //模组从该包装箱中移出，该模组变为未绑定BoxID的模组
        UpdateWrapper<WoPackagingBoxSubstrate> wrapper = new UpdateWrapper<>();
        wrapper.set(WoPackagingBoxSubstrate.PACKAGING_BOX_ID,null);
        wrapper.set(WoPackagingBoxSubstrate.SLOT,null);
        wrapper.eq(WoPackagingBoxSubstrate.ID,entity.getId());
        update(entity,wrapper);
//        entity.setPackagingBoxId("");
//        entity.setSlot(null);
//        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void insertSubstrate(ChangePackingBoxDTO dto) {
        WoPackagingBoxSubstrate entity = baseDao.selectById(dto.getId());
        //合箱写入包装箱拆合箱历史表
        ChangePackagingBoxHistory changePackagingBoxHistory = new ChangePackagingBoxHistory();
        changePackagingBoxHistory.setSubstrateId(dto.getWoSubstrateId());
        changePackagingBoxHistory.setTargetBoxNo(dto.getPackagingBoxId());
        changePackagingBoxHistory.setTargetSlot(dto.getSlot());
        changePackagingBoxHistory.setInputTime(LocalDateTime.now());
        changePackagingBoxHistory.setType(1);
        BaseSupportUtils.setCommonField(changePackagingBoxHistory);
        changePackagingBoxHistoryService.insert(changePackagingBoxHistory);
        //更新
        entity.setPackagingBoxId(dto.getPackagingBoxId());
        entity.setSlot(dto.getSlot());
        BaseSupportUtils.setUpdateCommonField(entity);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void replaceSubstrate(ChangePackingBoxDTO[] dtos) {
        for(ChangePackingBoxDTO dto : dtos){
            WoPackagingBoxSubstrate entity = baseDao.selectById(dto.getId());
            //换箱写入包装箱拆合箱历史表
            ChangePackagingBoxHistory changePackagingBoxHistory = new ChangePackagingBoxHistory();
            changePackagingBoxHistory.setSubstrateId(entity.getWoSubstrateId());
            changePackagingBoxHistory.setSourceBoxNo(entity.getPackagingBoxId());
            changePackagingBoxHistory.setSourceSlot(entity.getSlot());
            changePackagingBoxHistory.setTargetBoxNo(dto.getPackagingBoxId());
            changePackagingBoxHistory.setTargetSlot(dto.getSlot());
            changePackagingBoxHistory.setType(3);
            BaseSupportUtils.setCommonField(changePackagingBoxHistory);
            changePackagingBoxHistoryService.insert(changePackagingBoxHistory);
            //更新
            entity.setPackagingBoxId(dto.getPackagingBoxId());
            entity.setSlot(dto.getSlot());
            BaseSupportUtils.setUpdateCommonField(entity);
            updateById(entity);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void insertSubstrateManual(ChangePackingBoxDTO[] dtos) {
        for(ChangePackingBoxDTO dto : dtos) {
//            QueryWrapper<WoPackagingBoxSubstrate> wrapper = new QueryWrapper<>();
//            wrapper.eq(StringUtils.isNotBlank(dto.getWoSubstrateId()), WoPackagingBoxSubstrate.WO_SUBSTRATE_ID, dto.getWoSubstrateId());
            WoPackagingBoxSubstrate entity = ConvertUtils.sourceToTarget(dto, WoPackagingBoxSubstrate.class);
            BaseSupportUtils.setCommonField(entity);
            insert(entity);
            /*
            if (mapper.selectCount(wrapper) > 0 && StringUtils.isNotBlank(dto.getWoSubstrateId())) {
                WoPackagingBoxSubstrate substrate = mapper.selectList(wrapper).get(0);
                substrate.setPackagingBoxId(dto.getPackagingBoxId());
                BaseSupportUtils.setUpdateCommonField(substrate);
                updateById(substrate);
            } else {
                //模组ID有时为空，用户会输入”无码“
                if (dto.getWoSubstrateId() == null) {
                    dto.setWoSubstrateId("无码");
                }
                //效验数据
                ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);
                WoPackagingBoxSubstrate entity = ConvertUtils.sourceToTarget(dto, WoPackagingBoxSubstrate.class);
                BaseSupportUtils.setCommonField(entity);
                insert(entity);
            }
             */
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateSubstrateManual(ChangePackingBoxDTO[] dtos) {
        for(ChangePackingBoxDTO dto : dtos) {
            //判断模组id是否非空
            if(StringUtils.isBlank(dto.getWoSubstrateId())) {
                deleteById(dto.getId());
            }
            else {
                WoPackagingBoxSubstrate entity = ConvertUtils.sourceToTarget(dto, WoPackagingBoxSubstrate.class);
                BaseSupportUtils.setUpdateCommonField(entity);
                updateById(entity);
            }
        }
    }

    @Override
    public List<WoPackagingBoxSubstrateDTO> substrateList(Map<String, Object> params) {
        return mapper.substrateList(params);
    }

    @Override
    public PageData<WoPackagingBoxSubstrateDTO> substratePage(PackingInfoQueryParam param) {
        IPage<WoPackagingBoxSubstrateDTO> page = mapper.substratePage(
                new Page<>(param.getCurrent(), param.getSize()),
                param);
        return getPageData(page, WoPackagingBoxSubstrateDTO.class);
    }

    @Override
    public WoPackagingBoxSubstrate getBySubId(String subId) {
        return mapper.getOneByWoSubstrateId(subId);
    }

    @Override
    public int updatePackagingBoxIdAndSlotByWoSubstrateId(String packagingBoxId, Integer slot , String woSubstrateId) {
        return mapper.updatePackagingBoxIdAndSlotByWoSubstrateId(packagingBoxId,slot,woSubstrateId);
    }

    @Override
    public void setColor(List<WoPackagingBoxSubstrateDTO> list) {
        list.forEach(e -> {
            if(e.getSapMaterial()!=null){
                String[] sublist = e.getSapMaterial().split("\\.");
                if(sublist.length>2) {
                    switch (sublist[2]) {
                        case "C0001":
                            e.setColor("暗灰");
                            break;
                        case "C0002":
                            e.setColor("暗蓝");
                            break;
                        case "C0003":
                            e.setColor("暗绿");
                            break;
                        case "C0004":
                            e.setColor("暗黄");
                            break;
                        case "C0005":
                            e.setColor("暗橘黄");
                            break;
                        case "C0006":
                            e.setColor("暗红");
                            break;
                        case "C0007":
                            e.setColor("亮灰");
                            break;
                        case "C0008":
                            e.setColor("亮蓝");
                            break;
                        case "C0009":
                            e.setColor("亮绿");
                            break;
                        case "C0010":
                            e.setColor("亮黄");
                            break;
                        case "C0011":
                            e.setColor("亮橘黄");
                            break;
                        case "C0012":
                            e.setColor("亮红");
                            break;
                        case "C0013":
                            e.setColor("大红");
                            break;
                        case "C0014":
                            e.setColor("中国红1185");
                            break;
                        case "C0015":
                            e.setColor("新蓝825");
                            break;
                        case "C0016":
                            e.setColor("橄榄绿1186");
                            break;
                        case "C0017":
                            e.setColor("落日黄3726");
                            break;
                        case "C0018":
                            e.setColor("玛瑙灰4023");
                            break;
                        case "C0019":
                            e.setColor("中国红PV-748-R115");
                            break;
                        case "C0020":
                            e.setColor("阳光橙PV-748-0M2");
                            break;
                        case "C0021":
                            e.setColor("雪花银PV-748-W815");
                            break;
                        case "C0022":
                            e.setColor("苍穹灰PV-748-GR960");
                            break;
                        case "C0023":
                            e.setColor("阳光金PV-748-GL6020");
                            break;
                        case "C0024":
                            e.setColor("锦缎金PV-748-GL920");
                            break;
                        case "C0025":
                            e.setColor("森林绿PV-748-G830");
                            break;
                        case "C0026":
                            e.setColor("恒星绿PV-748-G602");
                            break;
                        case "C0027":
                            e.setColor("天空蓝PV-748-BL60");
                            break;
                        case "C0028":
                            e.setColor("香芋紫PV-748-BL40");
                            break;
                        case "C0029":
                            e.setColor("木槿紫PV-748-RZ92");
                            break;
                        case "C0030":
                            e.setColor("日光红PV-748-R602");
                            break;
                        case "C0031":
                            e.setColor("天空蓝BM0020");
                            break;
                        case "C0032":
                            e.setColor("天蓝色BM0101");
                            break;
                        case "C0033":
                            e.setColor("亮紫");
                            break;
                        case "C0034":
                            e.setColor("荒漠迷彩");
                            break;
                        case "C0035":
                            e.setColor("阳光金");
                            break;
                        case "C0036":
                            e.setColor("天空蓝");
                            break;
                        case "C0037":
                            e.setColor("翠绿色");
                            break;
                        case "C0038":
                            e.setColor("苍穹灰");
                            break;
                        case "C0039":
                            e.setColor("中国红-3");
                            break;
                        case "C0040":
                            e.setColor("301中国红亮彩");
                            break;
                        case "C0041":
                            e.setColor("淡金色");
                            break;
                        case "C0042":
                            e.setColor("土耳其条纹黄玉大理石");
                            break;
                        case "C0043":
                            e.setColor("意大利伯林顿灰大理石");
                            break;
                        case "C0044":
                            e.setColor("黑色");
                            break;
                        case "C0045":
                            e.setColor("50%透光黑色");
                            break;
                        default:
                            break;
                    }
                }
            }
        });
    }
}